@@ -112,7 +112,7 @@ typedef struct
 	SV *callback;
 	SV *userdata;
 	xchat_hook *hook;				  /* required for timers */
-
+	unsigned int depth;
 } HookData;
 
 static PerlInterpreter *my_perl = NULL;
@@ -215,7 +215,7 @@ timer_cb (void *userdata)
 			if (retVal == 0) {
 				/* if 0 is return the timer is going to get unhooked */
 				PUSHMARK (SP);
-				XPUSHs (sv_2mortal (newSVuv (PTR2UV (data->hook))));
+				XPUSHs (sv_2mortal (newSViv (PTR2IV (data->hook))));
 				PUTBACK;
 
 				call_pv ("Xchat::unhook", G_EVAL);
@@ -254,6 +254,9 @@ server_cb (char *word[], char *word_eol[], void *userdata)
 	ENTER;
 	SAVETMPS;
 
+	if (data->depth)
+		return XCHAT_EAT_NONE;
+
 	wd = newAV ();
 	sv_2mortal ((SV *) wd);
 	wd_eol = newAV ();
@@ -278,7 +281,9 @@ server_cb (char *word[], char *word_eol[], void *userdata)
 	XPUSHs (data->userdata);
 	PUTBACK;
 
+	data->depth++;
 	count = call_sv (data->callback, G_EVAL);
+	data->depth--;
 	SPAGAIN;
 	if (SvTRUE (ERRSV)) {
 		xchat_printf (ph, "Error in server callback %s", SvPV_nolen (ERRSV));
@@ -316,6 +321,9 @@ command_cb (char *word[], char *word_eol[], void *userdata)
 	ENTER;
 	SAVETMPS;
 
+	if (data->depth)
+		return XCHAT_EAT_NONE;
+
 	wd = newAV ();
 	sv_2mortal ((SV *) wd);
 	wd_eol = newAV ();
@@ -341,7 +349,9 @@ command_cb (char *word[], char *word_eol[], void *userdata)
 	XPUSHs (data->userdata);
 	PUTBACK;
 
+	data->depth++;
 	count = call_sv (data->callback, G_EVAL);
+	data->depth--;
 	SPAGAIN;
 	if (SvTRUE (ERRSV)) {
 		xchat_printf (ph, "Error in command callback %s", SvPV_nolen (ERRSV));
@@ -379,6 +389,9 @@ print_cb (char *word[], void *userdata)
 	ENTER;
 	SAVETMPS;
 
+	if (data->depth)
+		return XCHAT_EAT_NONE;
+
 	wd = newAV ();
 	sv_2mortal ((SV *) wd);
 
@@ -407,7 +420,9 @@ print_cb (char *word[], void *userdata)
 	XPUSHs (data->userdata);
 	PUTBACK;
 
+	data->depth++;
 	count = call_sv (data->callback, G_EVAL);
+	data->depth--;
 	SPAGAIN;
 	if (SvTRUE (ERRSV)) {
 		xchat_printf (ph, "Error in print callback %s", SvPV_nolen (ERRSV));
@@ -454,7 +469,7 @@ XS (XS_Xchat_register)
 		gui_entry = xchat_plugingui_add (ph, filename, name,
 													desc, version, NULL);
 
-		XSRETURN_UV (PTR2UV (gui_entry));
+		XSRETURN_IV (PTR2IV (gui_entry));
 
 	}
 }
@@ -527,7 +542,7 @@ XS (XS_Xchat_emit_print)
 
 		}
 
-		XSRETURN_UV (RETVAL);
+		XSRETURN_IV (RETVAL);
 	}
 }
 static
@@ -567,7 +582,7 @@ XS (XS_Xchat_get_prefs)
 			XSRETURN_PV (str);
 			break;
 		case 2:
-			XSRETURN_UV (integer);
+			XSRETURN_IV (integer);
 			break;
 		case 3:
 			if (integer) {
@@ -611,9 +626,10 @@ XS (XS_Xchat_hook_server)
 		SvREFCNT_inc (data->callback);
 		data->userdata = sv_mortalcopy (userdata);
 		SvREFCNT_inc (data->userdata);
+		data->depth = 0;
 		hook = xchat_hook_server (ph, name, pri, server_cb, data);
 
-		XSRETURN_UV (PTR2UV (hook));
+		XSRETURN_IV (PTR2IV (hook));
 	}
 }
 
@@ -651,9 +667,10 @@ XS (XS_Xchat_hook_command)
 		SvREFCNT_inc (data->callback);
 		data->userdata = sv_mortalcopy (userdata);
 		SvREFCNT_inc (data->userdata);
+		data->depth = 0;
 		hook = xchat_hook_command (ph, name, pri, command_cb, help_text, data);
 
-		XSRETURN_UV (PTR2UV (hook));
+		XSRETURN_IV (PTR2IV (hook));
 	}
 
 }
@@ -689,9 +706,10 @@ XS (XS_Xchat_hook_print)
 		SvREFCNT_inc (data->callback);
 		data->userdata = sv_mortalcopy (userdata);
 		SvREFCNT_inc (data->userdata);
+		data->depth = 0;
 		hook = xchat_hook_print (ph, name, pri, print_cb, data);
 
-		XSRETURN_UV (PTR2UV (hook));
+		XSRETURN_IV (PTR2IV (hook));
 	}
 }
 
@@ -728,7 +746,7 @@ XS (XS_Xchat_hook_timer)
 		hook = xchat_hook_timer (ph, timeout, timer_cb, data);
 		data->hook = hook;
 
-		XSRETURN_UV (PTR2UV (hook));
+		XSRETURN_IV (PTR2IV (hook));
 	}
 }
 
@@ -766,7 +784,7 @@ XS (XS_Xchat_hook_fd)
 		SvREFCNT_inc (data->userdata);
 		hook = xchat_hook_fd (ph, fd, flags, fd_cb, data);
 
-		XSRETURN_UV (PTR2UV (hook));
+		XSRETURN_IV (PTR2IV (hook));
 	}
 }
 
@@ -865,7 +883,7 @@ XS (XS_Xchat_find_context)
 		RETVAL = xchat_find_context (ph, server, chan);
 		if (RETVAL != NULL) {
 			/*                      xchat_print (ph, "XSUB - context found"); */
-			XSRETURN_UV (PTR2UV (RETVAL));
+			XSRETURN_IV (PTR2IV (RETVAL));
 		} else {
 			/*           xchat_print (ph, "XSUB - context not found"); */
 			XSRETURN_UNDEF;
@@ -880,7 +898,7 @@ XS (XS_Xchat_get_context)
 	if (items != 0) {
 		xchat_print (ph, "Usage: Xchat::get_context()");
 	} else {
-		XSRETURN_UV (PTR2UV (xchat_get_context (ph)));
+		XSRETURN_IV (PTR2IV (xchat_get_context (ph)));
 	}
 }
 
@@ -976,7 +994,7 @@ XS (XS_Xchat_get_list)
 				case 'p':
 					/*                                       xchat_printf (ph, "pointer: %s", fields[i]+1); */
 					hv_store (hash, fields[i] + 1, strlen (fields[i] + 1),
-								 newSVuv (PTR2UV (xchat_list_str (ph, list,
+								 newSViv (PTR2IV (xchat_list_str (ph, list,
 																			 fields[i] + 1)
 											 )), 0);
 					break;
