@@ -347,7 +347,7 @@ check_prefs_dir (void)
 #else
 		if (mkdir (dir, S_IRUSR | S_IWUSR | S_IXUSR) != 0)
 #endif
-			fe_message (_("Cannot create ~/.xchat2"), FALSE);
+			fe_message (_("Cannot create ~/.xchat2"), FE_MSG_ERROR);
 	}
 }
 
@@ -380,6 +380,7 @@ const struct prefs vars[] = {
 
 	{"completion_amount", P_OFFINT (completion_amount), TYPE_INT},
 	{"completion_auto", P_OFFINT (nickcompletion), TYPE_BOOL},
+	{"completion_sort", P_OFFINT (completion_sort), TYPE_INT},
 	{"completion_suffix", P_OFFSET (nick_suffix), TYPE_STR},
 
 	{"dcc_auto_chat", P_OFFINT (autodccchat), TYPE_INT},
@@ -421,9 +422,9 @@ const struct prefs vars[] = {
 	{"gui_dialog_width", P_OFFINT (dialog_width), TYPE_INT},
 	{"gui_hide_menu", P_OFFINT (hidemenu), TYPE_BOOL},
 	{"gui_input_style", P_OFFINT (style_inputbox), TYPE_BOOL},
+	{"gui_join_dialog", P_OFFINT (gui_join_dialog), TYPE_BOOL},
 	{"gui_lagometer", P_OFFINT (lagometer), TYPE_INT},
 	{"gui_mode_buttons", P_OFFINT (chanmodebuttons), TYPE_BOOL},
-	{"gui_slist_edit", P_OFFINT (slist_edit), TYPE_BOOL},
 	{"gui_slist_select", P_OFFINT (slist_select), TYPE_INT},
 	{"gui_slist_skip", P_OFFINT (slist_skip), TYPE_BOOL},
 	{"gui_throttlemeter", P_OFFINT (throttlemeter), TYPE_INT},
@@ -431,12 +432,12 @@ const struct prefs vars[] = {
 	{"gui_ulist_buttons", P_OFFINT (userlistbuttons), TYPE_BOOL},
 	{"gui_ulist_doubleclick", P_OFFSET (doubleclickuser), TYPE_STR},
 	{"gui_ulist_hide", P_OFFINT (hideuserlist), TYPE_BOOL},
-	{"gui_ulist_hilight_notify", P_OFFINT (hilitenotify), TYPE_BOOL},
-	{"gui_ulist_pos", P_OFFINT (paned_pos), TYPE_INT},
 	{"gui_ulist_resizable", P_OFFINT (paned_userlist), TYPE_BOOL},
 	{"gui_ulist_show_hosts", P_OFFINT(showhostname_in_userlist), TYPE_BOOL},
+	{"gui_ulist_size", P_OFFINT (paned_pos), TYPE_INT},
 	{"gui_ulist_sort", P_OFFINT (userlist_sort), TYPE_INT},
 	{"gui_ulist_style", P_OFFINT (style_namelistgad), TYPE_BOOL},
+	{"gui_usermenu", P_OFFINT (gui_usermenu), TYPE_BOOL},
 	{"gui_win_height", P_OFFINT (mainwindow_height), TYPE_INT},
 	{"gui_win_left", P_OFFINT (mainwindow_left), TYPE_INT},
 	{"gui_win_save", P_OFFINT (mainwindow_save), TYPE_BOOL},
@@ -462,15 +463,16 @@ const struct prefs vars[] = {
 	{"irc_conf_mode", P_OFFINT (confmode), TYPE_BOOL},
 	{"irc_extra_hilight", P_OFFSET (irc_extra_hilight), TYPE_STR},
 	{"irc_hide_version", P_OFFINT (hidever), TYPE_BOOL},
-	{"irc_id_ytext", P_OFFSET (irc_id_ytext), TYPE_STR},
 	{"irc_id_ntext", P_OFFSET (irc_id_ntext), TYPE_STR},
+	{"irc_id_ytext", P_OFFSET (irc_id_ytext), TYPE_STR},
 	{"irc_invisible", P_OFFINT (invisible), TYPE_BOOL},
+	{"irc_join_delay", P_OFFINT (irc_join_delay), TYPE_INT},
 	{"irc_logging", P_OFFINT (logging), TYPE_BOOL},
 	{"irc_logmask", P_OFFSET (logmask), TYPE_STR},
-	{"irc_nick_hilight", P_OFFSET (irc_nick_hilight), TYPE_STR},
 	{"irc_nick1", P_OFFSET (nick1), TYPE_STR},
 	{"irc_nick2", P_OFFSET (nick2), TYPE_STR},
 	{"irc_nick3", P_OFFSET (nick3), TYPE_STR},
+	{"irc_nick_hilight", P_OFFSET (irc_nick_hilight), TYPE_STR},
 	{"irc_no_hilight", P_OFFSET (irc_no_hilight), TYPE_STR},
 	{"irc_part_reason", P_OFFSET (partreason), TYPE_STR},
 	{"irc_quit_reason", P_OFFSET (quitreason), TYPE_STR},
@@ -481,6 +483,7 @@ const struct prefs vars[] = {
 	{"irc_user_name", P_OFFSET (username), TYPE_STR},
 	{"irc_wallops", P_OFFINT (wallops), TYPE_BOOL},
 	{"irc_who_join", P_OFFINT (userhost), TYPE_BOOL},
+	{"irc_whois_front", P_OFFINT (irc_whois_front), TYPE_BOOL},
 
 	{"net_auto_reconnect", P_OFFINT (autoreconnect), TYPE_BOOL},
 	{"net_auto_reconnectonfail", P_OFFINT (autoreconnectonfail), TYPE_BOOL},
@@ -496,7 +499,6 @@ const struct prefs vars[] = {
 	{"net_reconnect_delay", P_OFFINT (recon_delay), TYPE_INT},
 	{"net_throttle", P_OFFINT (throttle), TYPE_BOOL},
 
-	{"notify_color", P_OFFINT (nu_color), TYPE_INT},
 	{"notify_timeout", P_OFFINT (notify_timeout), TYPE_INT},
 	{"notify_whois_online", P_OFFINT (whois_on_notifyonline), TYPE_BOOL},
 
@@ -512,6 +514,8 @@ const struct prefs vars[] = {
 	{"tab_chans", P_OFFINT (tabchannels), TYPE_BOOL},
 	{"tab_dialogs", P_OFFINT (privmsgtab), TYPE_BOOL},
 	{"tab_dnd",  P_OFFINT (tab_dnd), TYPE_BOOL},
+	{"tab_icons", P_OFFINT (tab_icons), TYPE_BOOL},
+	{"tab_layout", P_OFFINT (tab_layout), TYPE_INT},
 	{"tab_new_to_front", P_OFFINT (newtabstofront), TYPE_INT},
 	{"tab_notices", P_OFFINT (notices_tabs), TYPE_BOOL},
 	{"tab_position", P_OFFINT (tabs_position), TYPE_INT},
@@ -531,7 +535,6 @@ const struct prefs vars[] = {
 	{"text_show_sep", P_OFFINT (show_separator), TYPE_BOOL},
 	{"text_stripcolor", P_OFFINT (stripcolor), TYPE_BOOL},
 	{"text_thin_sep", P_OFFINT (thin_separator), TYPE_BOOL},
-	{"text_tint", P_OFFINT (tint), TYPE_BOOL},
 	{"text_tint_blue", P_OFFINT (tint_blue), TYPE_INT},
 	{"text_tint_green", P_OFFINT (tint_green), TYPE_INT},
 	{"text_tint_red", P_OFFINT (tint_red), TYPE_INT},
@@ -584,6 +587,9 @@ load_config (void)
 
 	/* put in default values, anything left out is automatically zero */
 	prefs.local_ip = 0xffffffff;
+	prefs.irc_join_delay = 3;
+	prefs.show_marker = 1;
+	prefs.newtabstofront = 2;
 	prefs.completion_amount = 5;
 	prefs.away_timeout = 60;
 	prefs.away_size_max = 300;
@@ -608,13 +614,13 @@ load_config (void)
 	prefs.use_server_tab = 1;
 	prefs.privmsgtab = 1;
 	/*prefs.style_inputbox = 1;*/
-	prefs.nu_color = 4;
 	prefs.dccpermissions = 0600;
 	prefs.max_lines = 300;
 	prefs.mainwindow_width = 640;
 	prefs.mainwindow_height = 400;
 	prefs.dialog_width = 500;
 	prefs.dialog_height = 256;
+	prefs.gui_join_dialog = 1;
 	prefs.dcctimeout = 180;
 	prefs.dccstalltimeout = 60;
 	prefs.notify_timeout = 15;
@@ -668,8 +674,8 @@ load_config (void)
 			snprintf (prefs.dccdir, sizeof (prefs.dccdir), "%s\\Downloads", get_xdir_utf8 ());
 	}
 #else
-	sprintf (prefs.sounddir, "%s/sounds", get_xdir_utf8 ());
-	sprintf (prefs.dccdir, "%s/downloads", get_xdir_utf8 ());
+	snprintf (prefs.sounddir, sizeof (prefs.sounddir), "%s/sounds", get_xdir_utf8 ());
+	snprintf (prefs.dccdir, sizeof (prefs.dccdir), "%s/downloads", get_xdir_utf8 ());
 #endif
 	strcpy (prefs.doubleclickuser, "QUOTE WHOIS %s %s");
 	strcpy (prefs.awayreason, _("I'm busy"));
@@ -720,7 +726,7 @@ load_config (void)
 		/* OS/2 uses UID 0 all the time */
 		if (getuid () == 0)
 			fe_message (_("* Running IRC as root is stupid! You should\n"
-							"  create a User Account and use that to login.\n"), TRUE);
+							"  create a User Account and use that to login.\n"), FE_MSG_WARN|FE_MSG_WAIT);
 #endif
 #endif /* !WIN32 */
 
