@@ -65,6 +65,9 @@ extern SSL_CTX *ctx;				  /* xchat.c */
 static struct session *g_sess = NULL;
 #endif
 
+static GSList *away_list = NULL;
+GSList *serv_list = NULL;
+
 static void auto_reconnect (server *serv, int send_quit, int err);
 static void server_disconnect (session * sess, int sendquit, int err);
 static int server_cleanup (server * serv);
@@ -143,8 +146,8 @@ tcp_send_queue (server *serv)
 	if (!is_server (serv))
 		return 0;
 
-	/* first try priority 1 entries */
-	pri = 1;
+	/* try priority 2,1,0 */
+	pri = 2;
 	while (pri >= 0)
 	{
 		list = serv->outbound_queue;
@@ -200,14 +203,26 @@ tcp_send_len (server *serv, char *buf, int len)
 		return tcp_send_real (serv, buf, len);
 
 	dbuf = malloc (len + 2);	/* first byte is the priority */
-	dbuf[0] = 1;	/* pri 1 for most things */
+	dbuf[0] = 2;	/* pri 2 for most things */
 	memcpy (dbuf + 1, buf, len);
 	dbuf[len + 1] = 0;
 
 	/* privmsg and notice get a lower priority */
 	if (strncasecmp (dbuf + 1, "PRIVMSG", 7) == 0 ||
 		 strncasecmp (dbuf + 1, "NOTICE", 6) == 0)
-		dbuf[0] = 0;
+	{
+		dbuf[0] = 1;
+	}
+	else
+	{
+		/* WHO/MODE get the lowest priority */
+		if (strncasecmp (dbuf + 1, "WHO ", 4) == 0 ||
+		/* but only MODE queries, not changes */
+			(strncasecmp (dbuf + 1, "MODE", 4) == 0 &&
+			 strchr (dbuf, '-') == NULL &&
+			 strchr (dbuf, '+') == NULL))
+			dbuf[0] = 0;
+	}
 
 	serv->outbound_queue = g_slist_append (serv->outbound_queue, dbuf);
 	serv->sendq_len += len; /* tcp_send_queue uses strlen */
@@ -228,8 +243,9 @@ void
 tcp_sendf (server *serv, char *fmt, ...)
 {
 	va_list args;
-	/* keep this buffer in BSS */
-	static char send_buf[520];	/* good code hey (no it's not overflowable) */
+	/* keep this buffer in BSS. Converting UTF-8 to ISO-8859-x might make the
+      string shorter, so allow alot more than 512 for now. */
+	static char send_buf[1540];	/* good code hey (no it's not overflowable) */
 	int len;
 
 	va_start (args, fmt);
@@ -267,8 +283,8 @@ server_inline (server *serv, char *line, int len)
 	/* Checks whether we're set to use UTF-8 charset */
 	if ((serv->encoding == NULL && prefs.utf8_locale) ||
 	    (serv->encoding != NULL &&
-		 (strcasecmp (serv->encoding, "utf8") == 0 ||
-		  strcasecmp (serv->encoding, "utf-8") == 0)))
+		 (strcasecmp (serv->encoding, "UTF8") == 0 ||
+		  strcasecmp (serv->encoding, "UTF-8") == 0)))
 	{
 		/* The user has the UTF-8 charset set, either via /charset
 		command or from his UTF-8 locale. Thus, we first try the
@@ -306,6 +322,11 @@ server_inline (server *serv, char *line, int len)
 			conv_line[len] = 0;
 			conv_len = len;
 
+			/* if CP1255, convert it with the NUL terminator.
+				Works around SF bug #1122089 */
+			if (serv->using_cp1255)
+				conv_len++;
+
 			do
 			{
 				err = NULL;
@@ -313,7 +334,7 @@ server_inline (server *serv, char *line, int len)
 				utf_line_allocated = g_convert_with_fallback (conv_line, conv_len, "UTF-8", encoding, "?", &read_len, &utf_len, &err);
 				if (err != NULL)
 				{
-					if (err->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE)
+					if (err->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE && conv_len > (read_len + 1))
 					{
 						/* Make our best bet by removing the erroneous char.
 						   This will work for casual 8-bit strings with non-standard chars. */
@@ -333,6 +354,8 @@ server_inline (server *serv, char *line, int len)
 			{
 				line = utf_line_allocated;
 				len = utf_len;
+				if (serv->using_cp1255 && len > 0)
+					len--;
 			}
 			else
 			{
@@ -343,7 +366,7 @@ server_inline (server *serv, char *line, int len)
 	}
 
 	fe_add_rawlog (serv, line, len, FALSE);
-	url_check (line);
+	url_check_line (line, len);
 
 	/* let proto-irc.c handle it */
 	serv->p_inline (serv, line, len);
@@ -360,7 +383,6 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 	int sok = serv->sok;
 	int error, i, len;
 	char lbuf[2050];
-	char *temp;
 
 	while (1)
 	{
@@ -377,7 +399,7 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 			error = 0;
 			if (len < 0)
 			{
-				if (would_block_again ())
+				if (would_block ())
 					return TRUE;
 				error = sock_error ();
 			}
@@ -412,15 +434,7 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 
 			case '\n':
 				serv->linebuf[serv->pos] = 0;
-				if (prefs.stripcolor)
-				{
-					temp = strip_color (serv->linebuf);
-					server_inline (serv, temp, strlen (temp));
-					free (temp);
-				} else
-				{
-					server_inline (serv, serv->linebuf, serv->pos);
-				}
+				server_inline (serv, serv->linebuf, serv->pos);
 				serv->pos = 0;
 				break;
 
@@ -452,10 +466,12 @@ server_connected (server * serv)
 		if (serv->network)
 		{
 			serv->p_login (serv,
-								(((ircnet *)serv->network)->user) ?
+								(!(((ircnet *)serv->network)->flags & FLAG_USE_GLOBAL) &&
+								 (((ircnet *)serv->network)->user)) ?
 								(((ircnet *)serv->network)->user) :
 								prefs.username,
-								(((ircnet *)serv->network)->real) ?
+								(!(((ircnet *)serv->network)->flags & FLAG_USE_GLOBAL) &&
+								 (((ircnet *)serv->network)->real)) ?
 								(((ircnet *)serv->network)->real) :
 								prefs.realname);
 		} else
@@ -468,7 +484,8 @@ server_connected (server * serv)
 						 NULL, NULL, 0);
 	}
 
-	set_server_name (serv, serv->servername);
+	server_set_name (serv, serv->servername);
+	fe_server_event (serv, FE_SE_CONNECT, 0);
 }
 
 #ifdef WIN32
@@ -493,6 +510,12 @@ server_stopconnecting (server * serv)
 		serv->iotag = 0;
 	}
 
+	if (serv->joindelay_tag)
+	{
+		fe_timeout_remove (serv->joindelay_tag);
+		serv->joindelay_tag = 0;
+	}
+
 #ifndef WIN32
 	/* kill the child process trying to connect */
 	kill (serv->childpid, SIGKILL);
@@ -523,6 +546,7 @@ server_stopconnecting (server * serv)
 	fe_progressbar_end (serv);
 
 	serv->connecting = FALSE;
+	fe_server_event (serv, FE_SE_DISCONNECT, 0);
 }
 
 #ifdef USE_OPENSSL
@@ -772,6 +796,7 @@ auto_reconnect (server *serv, int send_quit, int err)
 		serv->reconnect_away = serv->is_away;
 
 	serv->recondelay_tag = fe_timeout_add (del, timeout_auto_reconnect, serv);
+	fe_server_event (serv, FE_SE_RECONDELAY, del);
 }
 
 static void
@@ -965,6 +990,12 @@ server_cleanup (server * serv)
 		serv->iotag = 0;
 	}
 
+	if (serv->joindelay_tag)
+	{
+		fe_timeout_remove (serv->joindelay_tag);
+		serv->joindelay_tag = 0;
+	}
+
 #ifdef USE_OPENSSL
 	if (serv->ssl)
 	{
@@ -1008,12 +1039,16 @@ server_disconnect (session * sess, int sendquit, int err)
 	GSList *list;
 	char tbuf[64];
 
+	joind (1, serv);
+
 	/* send our QUIT reason */
 	if (sendquit && serv->connected)
 	{
 		server_sendquit (sess);
 	}
 
+	fe_server_event (serv, FE_SE_DISCONNECT, 0);
+
 	/* close all sockets & io tags */
 	switch (server_cleanup (serv))
 	{
@@ -1249,7 +1284,7 @@ http_read_line (int print_fd, int sok, char *buf, int len)
 {
 #ifdef WIN32
 	/* make sure waitline() uses recv() or it'll fail on win32 */
-	len = waitline2 (source, buf, len);
+	len = waitline (sok, buf, len, FALSE);
 #else
 	len = waitline (sok, buf, len, TRUE);
 #endif
@@ -1498,11 +1533,12 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	safe_strcpy (serv->servername, hostname, sizeof (serv->servername));
 	safe_strcpy (serv->hostname, hostname, sizeof (serv->hostname));
 
-	set_server_defaults (serv);
+	server_set_defaults (serv);
 	serv->connecting = TRUE;
 	serv->port = port;
 	serv->no_login = no_login;
 
+	fe_server_event (serv, FE_SE_CONNECTING, 0);
 	fe_set_away (serv);
 	server_flush_queue (serv);
 
@@ -1527,6 +1563,9 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 										(LPTHREAD_START_ROUTINE)server_child,
 										serv, 0, (DWORD *)&pid));
 #else
+#ifdef LOOKUPD
+	rand();	/* CL: net_resolve calls rand() when LOOKUPD is set, so prepare a different seed for each child. This method giver a bigger variation in seed values than calling srand(time(0)) in the child itself. */
+#endif
 	switch (pid = fork ())
 	{
 	case -1:
@@ -1540,11 +1579,12 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	}
 #endif
 	serv->childpid = pid;
-	serv->iotag =
-		fe_input_add (serv->childread, FIA_READ|FIA_FD, server_read_child, serv);
+	serv->iotag = fe_input_add (serv->childread, FIA_READ, server_read_child,
+										 serv);
 }
 
-void server_fill_her_up (server *serv)
+void
+server_fill_her_up (server *serv)
 {
 	serv->connect = server_connect;
 	serv->disconnect = server_disconnect;
@@ -1554,3 +1594,229 @@ void server_fill_her_up (server *serv)
 
 	proto_fill_her_up (serv);
 }
+
+void
+server_set_encoding (server *serv, char *new_encoding)
+{
+	char *space;
+
+	if (serv->encoding)
+	{
+		free (serv->encoding);
+		/* can be left as NULL to indicate system encoding */
+		serv->encoding = NULL;
+		serv->using_cp1255 = FALSE;
+	}
+
+	if (new_encoding)
+	{
+		serv->encoding = strdup (new_encoding);
+		/* the serverlist GUI might have added a space 
+			and short description - remove it. */
+		space = strchr (serv->encoding, ' ');
+		if (space)
+			space[0] = 0;
+
+		/* server_inline() uses this flag */
+		if (!strcasecmp (serv->encoding, "CP1255") ||
+			 !strcasecmp (serv->encoding, "WINDOWS-1255"))
+			serv->using_cp1255 = TRUE;
+	}
+}
+
+server *
+server_new (void)
+{
+	static int id = 0;
+	server *serv;
+
+	serv = malloc (sizeof (struct server));
+	memset (serv, 0, sizeof (struct server));
+
+	/* use server.c and proto-irc.c functions */
+	server_fill_her_up (serv);
+
+	serv->id = id++;
+	serv->sok = -1;
+	strcpy (serv->nick, prefs.nick1);
+	server_set_defaults (serv);
+
+	serv_list = g_slist_prepend (serv_list, serv);
+
+	fe_new_server (serv);
+
+	return serv;
+}
+
+int
+is_server (server *serv)
+{
+	return g_slist_find (serv_list, serv) ? 1 : 0;
+}
+
+void
+server_set_defaults (server *serv)
+{
+	if (serv->chantypes)
+		free (serv->chantypes);
+	if (serv->chanmodes)
+		free (serv->chanmodes);
+	if (serv->nick_prefixes)
+		free (serv->nick_prefixes);
+	if (serv->nick_modes)
+		free (serv->nick_modes);
+
+	serv->chantypes = strdup ("#&!+");
+	serv->chanmodes = strdup ("beI,k,l");
+	serv->nick_prefixes = strdup ("@%+");
+	serv->nick_modes = strdup ("ohv");
+
+	serv->nickcount = 1;
+	serv->nickservtype = 0;
+	serv->end_of_motd = FALSE;
+	serv->is_away = FALSE;
+	serv->supports_watch = FALSE;
+	serv->bad_prefix = FALSE;
+	serv->use_who = TRUE;
+	serv->have_idmsg = FALSE;
+	serv->have_except = FALSE;
+}
+
+char *
+server_get_network (server *serv, gboolean fallback)
+{
+	if (serv->network)
+		return ((ircnet *)serv->network)->name;
+
+	if (fallback)
+		return serv->servername;
+
+	return NULL;
+}
+
+void
+server_set_name (server *serv, char *name)
+{
+	GSList *list = sess_list;
+	session *sess;
+
+	if (name[0] == 0)
+		name = serv->hostname;
+
+	/* strncpy parameters must NOT overlap */
+	if (name != serv->servername)
+	{
+		safe_strcpy (serv->servername, name, sizeof (serv->servername));
+	}
+
+	while (list)
+	{
+		sess = (session *) list->data;
+		if (sess->server == serv)
+			fe_set_title (sess);
+		list = list->next;
+	}
+
+	if (serv->server_session->type == SESS_SERVER)
+	{
+		if (serv->network)
+		{
+			safe_strcpy (serv->server_session->channel, ((ircnet *)serv->network)->name, CHANLEN);
+		} else
+		{
+			safe_strcpy (serv->server_session->channel, name, CHANLEN);
+		}
+		fe_set_channel (serv->server_session);
+	}
+}
+
+struct away_msg *
+server_away_find_message (server *serv, char *nick)
+{
+	struct away_msg *away;
+	GSList *list = away_list;
+	while (list)
+	{
+		away = (struct away_msg *) list->data;
+		if (away->server == serv && !serv->p_cmp (nick, away->nick))
+			return away;
+		list = list->next;
+	}
+	return NULL;
+}
+
+static void
+server_away_free_messages (server *serv)
+{
+	GSList *list, *next;
+	struct away_msg *away;
+
+	list = away_list;
+	while (list)
+	{
+		away = list->data;
+		next = list->next;
+		if (away->server == serv)
+		{
+			away_list = g_slist_remove (away_list, away);
+			if (away->message)
+				free (away->message);
+			free (away);
+			next = away_list;
+		}
+		list = next;
+	}
+}
+
+void
+server_away_save_message (server *serv, char *nick, char *msg)
+{
+	struct away_msg *away = server_away_find_message (serv, nick);
+
+	if (away)						  /* Change message for known user */
+	{
+		if (away->message)
+			free (away->message);
+		away->message = strdup (msg);
+	} else
+		/* Create brand new entry */
+	{
+		away = malloc (sizeof (struct away_msg));
+		if (away)
+		{
+			away->server = serv;
+			safe_strcpy (away->nick, nick, sizeof (away->nick));
+			away->message = strdup (msg);
+			away_list = g_slist_prepend (away_list, away);
+		}
+	}
+}
+
+void
+server_free (server *serv)
+{
+	serv->cleanup (serv);
+
+	serv_list = g_slist_remove (serv_list, serv);
+
+	dcc_notify_kill (serv);
+	serv->flush_queue (serv);
+	server_away_free_messages (serv);
+
+	free (serv->nick_modes);
+	free (serv->nick_prefixes);
+	free (serv->chanmodes);
+	free (serv->chantypes);
+	if (serv->bad_nick_prefixes)
+		free (serv->bad_nick_prefixes);
+	if (serv->last_away_reason)
+		free (serv->last_away_reason);
+	if (serv->encoding)
+		free (serv->encoding);
+
+	fe_server_callback (serv);
+
+	free (serv);
+
+	notify_cleanup ();
+}
