@@ -24,11 +24,13 @@
 #include <unistd.h>
 
 #include "xchat.h"
+#include <glib/ghash.h>
 
 #include "cfgfiles.h"
 #include "fe.h"
 #include "server.h"
 #include "text.h"
+#include "util.h" /* token_foreach */
 #include "xchatc.h"
 
 #include "servlist.h"
@@ -39,14 +41,13 @@ struct defaultserver
 	char *network;
 	char *host;
 	char *channel;
+	char *charset;
 };
 
 static const struct defaultserver def[] =
 {
 	{"2600net",	0},
 	{0,			"irc.2600.net"},
-	{0,			"ptolomea.2600.net"},
-	{0,			"collective.2600.net"},
 
 	{"AccessIRC",	0},
 	{0,			"irc.accessirc.net"},
@@ -118,6 +119,9 @@ static const struct defaultserver def[] =
 	{0,			"irc.azzurra.org"},
 	{0,			"crypto.azzurra.org"},
 
+	{"Beirut", 0},
+	{0,			"irc.beirut.com"},
+
 	{"Buzzpot", 0},
 	{0,			"irc.chatspike.net"},
 
@@ -138,6 +142,10 @@ static const struct defaultserver def[] =
 	{0,			"unix.coolchat.net"},
 	{0,			"toronto.coolchat.net"},
 
+	{"Criten", 0},
+	{0,			"irc.criten.net"},
+	{0,			"irc.eu.criten.net"},
+
 	{"DALnet",	0},
 	{0,			"irc.dal.net"},
 	{0,			"irc.eu.dal.net"},
@@ -159,10 +167,12 @@ static const struct defaultserver def[] =
 	{0,			"irc.lagged.org"},
 	{0,			"irc.lightning.net"},
 	{0,			"irc.mindspring.com"},
-	{0,			"irc.rt.ru"},
 	{0,			"irc.easynews.com"},
 	{0,			"irc.servercentral.net"},
 
+	{"EnterTheGame",	0},
+	{0,			"IRC.EnterTheGame.Com"},
+
 	{"EUIrc",	0},
 	{0,			"irc.euirc.net"},
 	{0,			"irc.ham.de.euirc.net"},
@@ -186,7 +196,6 @@ static const struct defaultserver def[] =
 
 	{"FEFNet",	0},
 	{0,			"irc.fef.net"},
-	{0,			"irc.villagenet.com"},
 	{0,			"irc.ggn.net"},
 	{0,			"irc.vendetta.com"},
 
@@ -226,6 +235,9 @@ static const struct defaultserver def[] =
 	{"Hashmark",	0},
 	{0,			"irc.hashmark.net"},
 
+	{"IdleMonkeys", 0},
+	{0,			"irc.idlemonkeys.net"},
+
 	{"Infinity-IRC",	0},
 	{0,			"Atlanta.GA.US.Infinity-IRC.Org"},
 	{0,			"Babylon.NY.US.Infinity-IRC.Org"},
@@ -244,26 +256,25 @@ static const struct defaultserver def[] =
 	{0,			"tonsberg.no.eu.irclink.net"},
 
 	{"IRCNet",		0},
-	{0,				"irc.stealth.net/5550"},
+	{0,				"irc.ircnet.com"},
+	{0,				"irc.stealth.net/6668"},
 	{0,				"ircnet.demon.co.uk"},
 	{0,				"ircnet.hinet.hr"},
 	{0,				"irc.datacomm.ch"},
 	{0,				"ircnet.kaptech.fr"},
-	{0,				"irc.flashnet.it"},
-	{0,				"irc.cwitaly.it"},
 	{0,				"ircnet.easynet.co.uk"},
 	{0,				"random.ircd.de"},
 	{0,				"ircnet.netvision.net.il"},
 	{0,				"irc.seed.net.tw"},
 	{0,				"irc.cs.hut.fi"},
 
-	{"IRCSoulZ",	0},
-	{0,			"irc.ircsoulz.net"},
-
 	{"Irctoo.net",	0},
 	{0,			"irc.canadian.net"},
 	{0,			"irc.irctoo.net"},
 
+	{"Krstarica", 0},
+	{0,			"irc.krstarica.com"},
+
 	{"Librenet",	0},
 	{0,			"irc.librenet.net"},
 	{0,			"ielf.fr.librenet.net"},
@@ -282,6 +293,9 @@ static const struct defaultserver def[] =
 	{"Majistic",	0},
 	{0,			"irc.majistic.net"},
 
+	{"MIXXnet",		0},
+	{0,			"irc.mixxnet.net"},
+
 	{"NeverNET",	0},
 	{0,			"irc.nevernet.net"},
 	{0,			"imagine.nevernet.net"},
@@ -305,12 +319,12 @@ static const struct defaultserver def[] =
 	{"NullusNet",	0},
 	{0,			"irc.nullus.net"},
 
+	{"Oceanius", 0},
+	{0,			"irc.oceanius.com"},
+
 	{"OFTC",	0},
 	{0,			"irc.oftc.net"},
 
-	{"OnlyIRC",		0},
-	{0,			"irc.onlyirc.net"},
-
 	{"OtherNet",	0},
 	{0,			"irc.othernet.org"},
 
@@ -318,12 +332,6 @@ static const struct defaultserver def[] =
 	{0,			"sydney.oz.org"},
 	{0,			"melbourne.oz.org"},
 
-	{"Progameplayer",	0},
-	{0,			"irc.progameplayer.com"},
-	{0,			"melancholia.oh.us.progameplayer.com"},
-	{0,			"deimos.oh.us.progameplayer.com"},
-	{0,			"paradigm.oh.us.progameplayer.com"},
-
 	{"PTlink",	0},
 	{0,			"irc.PTlink.net"},
 	{0,			"aaia.PTlink.net"},
@@ -359,9 +367,6 @@ static const struct defaultserver def[] =
 	{0,			"iscte.PTnet.org"},
 	{0,			"ubi.PTnet.org"},
 
-	{"QChat.net",	0},
-	{0,			"irc.qchat.net"},
-
 	{"QuakeNet",	0},
 	{0,			"irc.quakenet.org"},
 	{0,			"irc.se.quakenet.org"},
@@ -373,11 +378,6 @@ static const struct defaultserver def[] =
 	{0,			"irc.de.quakenet.org"},
 	{0,			"irc.it.quakenet.org"},
 
-	{"RealmNET",	0},
-	{0,			"irc.realmnet.com"},
-	{0,			"irc.eu.realmnet.com"},
-	{0,			"irc.us.realmnet.com"},
-
 	{"RebelChat",	0},
 	{0,			"irc.rebelchat.org"},
 
@@ -391,9 +391,6 @@ static const struct defaultserver def[] =
 	{0,			"garbage.recycled-irc.org"},
 	{0,			"dustbin.recycled-irc.org"},
 
-	{"RedeBrasilIRC",	0},
-	{0,			"irc.redebrasilirc.org"},
-
 	{"RizeNET", 0},
 	{0,			"irc.rizenet.org"},
 	{0,			"omega.rizenet.org"},
@@ -401,6 +398,13 @@ static const struct defaultserver def[] =
 	{0,			"lisa.rizenet.org"},
 	{0,			"scott.rizenet.org"},
 
+	{"RusNet", 0, 0, "KOI8-R (Cyrillic)"},
+	{0,			"irc.tomsk.net"},
+	{0,			"irc.rinet.ru"},
+	{0,			"irc.run.net"},
+	{0,			"irc.ru"},
+	{0,			"irc.lucky.net"},
+
 	{"SceneNet",	0},
 	{0,			"irc.scene.org"},
 	{0,			"irc.eu.scene.org"},
@@ -434,18 +438,9 @@ static const struct defaultserver def[] =
 	{0,			"tahoma.starchat.net"},
 	{0,			"neo.starchat.net"},
 
-	{"SubCultNet",	0},
-	{0,			"irc.subcult.ch"},
-	{0,			"irc.phuncrew.ch"},
-	{0,			"irc.mgz.ch"},
-	{0,			"irc.nazgul.ch/6666"},
-
 	{"TNI3",			0},
 	{0,			"irc.tni3.com"},
 
-	{"TopGamers",	0},
-	{0,			"irc.topgamers.net"},
-
 	{"UnderNet",	0},
 	{0,			"us.undernet.org"},
 	{0,			"eu.undernet.org"},
@@ -458,8 +453,8 @@ static const struct defaultserver def[] =
 	{0,			"irc.telecoms.bg"},
 	{0,			"irc.tu-varna.edu"},
 
-	{"Unsecurity",	0},
-	{0,			"irc.unsecurity.org"},
+	{"Whiffle",	0},
+	{0,			"irc.whiffle.org"},
 
 	{"Worldnet",		0},
 	{0,			"irc.worldnet.net"},
@@ -487,7 +482,7 @@ GSList *network_list = 0;
 
 
 void
-servlist_connect (session *sess, ircnet *net)
+servlist_connect (session *sess, ircnet *net, gboolean join)
 {
 	ircserver *ircserv;
 	GSList *list;
@@ -508,12 +503,16 @@ servlist_connect (session *sess, ircnet *net)
 	/* incase a protocol switch is added to the servlist gui */
 	server_fill_her_up (sess->server);
 
-	sess->willjoinchannel[0] = 0;
-	serv->password[0] = 0;
+	if (join)
+	{
+		sess->willjoinchannel[0] = 0;
 
-	if (net->autojoin)
-		safe_strcpy (sess->willjoinchannel, net->autojoin,
-						 sizeof (sess->willjoinchannel));
+		if (net->autojoin)
+			safe_strcpy (sess->willjoinchannel, net->autojoin,
+							 sizeof (sess->willjoinchannel));
+	}
+
+	serv->password[0] = 0;
 	if (net->pass)
 		safe_strcpy (serv->password, net->pass, sizeof (serv->password));
 
@@ -572,7 +571,7 @@ servlist_connect_by_netname (session *sess, char *network)
 
 		if (strcasecmp (net->name, network) == 0)
 		{
-			servlist_connect (sess, net);
+			servlist_connect (sess, net, FALSE);
 			return 1;
 		}
 
@@ -615,7 +614,7 @@ servlist_auto_connect (session *sess)
 
 		if (net->flags & FLAG_AUTO_CONNECT)
 		{
-			servlist_connect (sess, net);
+			servlist_connect (sess, net, TRUE);
 			ret = 1;
 		}
 
@@ -630,7 +629,7 @@ servlist_cycle_cb (server *serv)
 {
 	PrintTextf (serv->server_session,
 		_("Cycling to next server in %s...\n"), ((ircnet *)serv->network)->name);
-	servlist_connect (serv->server_session, serv->network);
+	servlist_connect (serv->server_session, serv->network, TRUE);
 
 	return 0;
 }
@@ -662,7 +661,7 @@ servlist_cycle (server *serv)
 			if (del)
 				serv->recondelay_tag = fe_timeout_add (del, servlist_cycle_cb, serv);
 			else
-				servlist_connect (serv->server_session, net);
+				servlist_connect (serv->server_session, net, TRUE);
 
 			return TRUE;
 		}
@@ -694,8 +693,38 @@ servlist_server_find (ircnet *net, char *name, int *pos)
 	return NULL;
 }
 
+/* find a network (e.g. (ircnet *) to "FreeNode") from a hostname
+   (e.g. "irc.eu.freenode.net") */
+
 ircnet *
-servlist_net_find (char *name, int *pos)
+servlist_net_find_from_server (char *server_name)
+{
+	GSList *list = network_list;
+	GSList *slist;
+	ircnet *net;
+	ircserver *serv;
+
+	while (list)
+	{
+		net = list->data;
+
+		slist = net->servlist;
+		while (slist)
+		{
+			serv = slist->data;
+			if (strcasecmp (serv->hostname, server_name) == 0)
+				return net;
+			slist = slist->next;
+		}
+
+		list = list->next;
+	}
+
+	return NULL;
+}
+
+ircnet *
+servlist_net_find (char *name, int *pos, int (*cmpfunc) (const char *, const char *))
 {
 	GSList *list = network_list;
 	ircnet *net;
@@ -704,7 +733,7 @@ servlist_net_find (char *name, int *pos)
 	while (list)
 	{
 		net = list->data;
-		if (strcmp (net->name, name) == 0)
+		if (cmpfunc (net->name, name) == 0)
 		{
 			if (pos)
 				*pos = i;
@@ -762,6 +791,8 @@ servlist_net_remove (ircnet *net)
 
 	if (net->nick)
 		free (net->nick);
+	if (net->nick2)
+		free (net->nick2);
 	if (net->user)
 		free (net->user);
 	if (net->real)
@@ -829,7 +860,12 @@ servlist_load_defaults (void)
 #endif
 			if (def[i].channel)
 				net->autojoin = strdup (def[i].channel);
-			if (!strcmp (def[i].network, "ChatJunkies"))
+			if (def[i].charset)
+			{
+				free (net->encoding);
+				net->encoding = strdup (def[i].charset);
+			}
+			if (g_str_hash (def[i].network) == 0x8e1b96f7)
 				prefs.slist_select = j;
 			j++;
 		} else
@@ -868,6 +904,9 @@ servlist_load (void)
 			case 'I':
 				net->nick = strdup (buf + 2);
 				break;
+			case 'i':
+				net->nick2 = strdup (buf + 2);
+				break;
 			case 'U':
 				net->user = strdup (buf + 2);
 				break;
@@ -957,41 +996,6 @@ servlist_write_ccmd (char *str, void *fp)
 	return fprintf (fp, "C=%s\n", (str[0] == '/') ? str + 1 : str);
 }
 
-/* separates a string according to a 'sep' char, then calls the callback
-   function for each token found */
-
-int
-token_foreach (char *str, char sep,
-					int (*callback) (char *str, void *ud), void *ud)
-{
-	char t, *start = str;
-
-	while (1)
-	{
-		if (*str == sep || *str == 0)
-		{
-			t = *str;
-			*str = 0;
-			if (callback (start, ud) < 1)
-			{
-				*str = t;
-				return FALSE;
-			}
-			*str = t;
-
-			if (*str == 0)
-				break;
-			str++;
-			start = str;
-
-		} else
-		{
-			str += g_utf8_skip[*((unsigned char *)str)];
-		}
-	}
-
-	return TRUE;
-}
 
 int
 servlist_save (void)
@@ -1025,6 +1029,8 @@ servlist_save (void)
 		fprintf (fp, "N=%s\n", net->name);
 		if (net->nick)
 			fprintf (fp, "I=%s\n", net->nick);
+		if (net->nick2)
+			fprintf (fp, "i=%s\n", net->nick2);
 		if (net->user)
 			fprintf (fp, "U=%s\n", net->user);
 		if (net->real)
@@ -1043,7 +1049,7 @@ servlist_save (void)
 			{
 				snprintf (buf, sizeof (buf), _("Warning: \"%s\" character set is unknown. No conversion will be applied for network %s."),
 							 net->encoding, net->name);
-				fe_message (buf, FALSE);
+				fe_message (buf, FE_MSG_WARN);
 			}
 		}
 
