@@ -51,6 +51,11 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define FILEPATH_LEN_MAX MAXPATHLEN
 #endif
 
+/* force a 32bit CMP.L */
+#define CMPL(a, c0, c1, c2, c3) (a == (guint32)(c0 | (c1 << 8) | (c2 << 16) | (c3 << 24)))
+#define WORDL(c0, c1, c2, c3) (guint32)(c0 | (c1 << 8) | (c2 << 16) | (c3 << 24))
+#define WORDW(c0, c1) (guint16)(c0 | (c1 << 8))
+
 #ifdef WIN32						/* for win32 */
 #define OFLAGS O_BINARY
 #define sleep(t) _sleep(t*1000)
@@ -115,9 +120,6 @@ struct xchatprefs
 	char quitreason[256];
 	char partreason[256];
 	char font_normal[FONTNAMELEN + 1];
-#ifdef USE_ZVT
-	char font_shell[FONTNAMELEN + 1];
-#endif
 	char doubleclickuser[256];
 	char sounddir[PATHLEN + 1];
 	char soundcmd[PATHLEN + 1];
@@ -154,6 +156,7 @@ struct xchatprefs
 	int away_size_max;
 	int paned_pos;
 	int tabs_position;
+	int tab_layout;
 	int max_auto_indent;
 	int dcc_blocksize;
 	int max_lines;
@@ -168,7 +171,10 @@ struct xchatprefs
 	int mainwindow_top;
 	int mainwindow_width;
 	int mainwindow_height;
+	int completion_sort;
 	int gui_win_state;
+	int gui_usermenu;
+	int gui_join_dialog;
 	int dialog_left;
 	int dialog_top;
 	int dialog_width;
@@ -177,18 +183,17 @@ struct xchatprefs
 	int recon_delay;
 	int bantype;
 	int userlist_sort;
-	int nu_color;
-	unsigned long local_ip;
-	unsigned long dcc_ip;
+	guint32 local_ip;
+	guint32 dcc_ip;
 	char dcc_ip_str[DOMAINLEN + 1];
 
 	unsigned int tab_small;
 	unsigned int tab_dnd;
 	unsigned int tab_sort;
+	unsigned int tab_icons;
 	unsigned int mainwindow_save;
 	unsigned int perc_color;
 	unsigned int perc_ascii;
-	unsigned int use_trans;
 	unsigned int autosave;
 	unsigned int autodialog;
 	unsigned int autosave_url;
@@ -214,14 +219,12 @@ struct xchatprefs
 	unsigned int autoopendccrecvwindow;
 	unsigned int autoopendccchatwindow;
 	unsigned int transparent;
-	unsigned int tint;
 	unsigned int stripcolor;
 	unsigned int timestamp;
 	unsigned int fastdccsend;
 	unsigned int dcc_send_fillspaces;
 	unsigned int dcc_remove;
 	unsigned int slist_skip;
-	unsigned int slist_edit;
 	unsigned int slist_select;
 	unsigned int filterbeep;
 	unsigned int beepmsg;
@@ -230,11 +233,11 @@ struct xchatprefs
 	unsigned int flash_hilight;
 	unsigned int truncchans;
 	unsigned int privmsgtab;
+	unsigned int irc_join_delay;
 	unsigned int logging;
 	unsigned int timestamp_logs;
 	unsigned int newtabstofront;
 	unsigned int dccwithnick;
-	unsigned int hilitenotify;
 	unsigned int hidever;
 	unsigned int ip_from_server;
 	unsigned int raw_modes;
@@ -243,6 +246,7 @@ struct xchatprefs
 	unsigned int auto_unmark_away;
 	unsigned int away_track;
 	unsigned int userhost;
+	unsigned int irc_whois_front;
 	unsigned int use_server_tab;
 	unsigned int notices_tabs;
 	unsigned int style_namelistgad;
@@ -262,7 +266,6 @@ struct xchatprefs
 	unsigned int perlwarnings;
 	unsigned int lagometer;
 	unsigned int throttlemeter;
-	unsigned int hebrew;
 	unsigned int pingtimeout;
 	unsigned int whois_on_notifyonline;
 	unsigned int wait_on_exit;
@@ -357,7 +360,8 @@ typedef struct server
 	void (*p_quit)(struct server *, char *reason);
 	void (*p_kick)(struct server *, char *channel, char *nick, char *reason);
 	void (*p_part)(struct server *, char *channel, char *reason);
-	void (*p_nickserv)(struct server *, char *pass);
+	void (*p_ns_identify)(struct server *, char *pass);
+	void (*p_ns_ghost)(struct server *, char *usname, char *pass);
 	void (*p_join)(struct server *, char *channel, char *key);
 	void (*p_login)(struct server *, char *user, char *realname);
 	void (*p_join_info)(struct server *, char *channel);
@@ -395,6 +399,7 @@ typedef struct server
 	int childpid;
 	int iotag;
 	int recondelay_tag;				/* reconnect delay timeout */
+	int joindelay_tag;				/* waiting before we send JOIN */
 	char hostname[128];				/* real ip number */
 	char servername[128];			/* what the server says is its name */
 	char password[86];
@@ -493,4 +498,7 @@ struct popup
 	char *name;
 };
 
+/* CL: get a random int in the range [0..n-1]. DON'T use rand() % n, it gives terrible results. */
+#define RAND_INT(n) ((int)(rand() / (RAND_MAX + 1.0) * (n)))
+
 #endif
