@@ -43,6 +43,7 @@
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
 #include "../common/cfgfiles.h"
+#include "../common/fe.h"
 #include "../common/userlist.h"
 #include "../common/outbound.h"
 #include "../common/util.h"
@@ -191,8 +192,8 @@ key_init ()
 	{
 		key_load_defaults ();
 		if (key_load_kbs (NULL) == 1)
-			gtkutil_simpledialog
-				(_("There was an error loading key bindings configuration"));
+			fe_message (_("There was an error loading key"
+							" bindings configuration"), FE_MSG_ERROR);
 	}
 }
 
@@ -697,7 +698,7 @@ key_dialog_show ()
 	}
 
 	key_dialog =
-			  mg_create_generic_tab ("editkeys", _("X-Chat: Keyboard Shortcuts"),
+			  mg_create_generic_tab ("editkeys", _("XChat: Keyboard Shortcuts"),
 							TRUE, FALSE, key_dialog_close, NULL, 560, 330, &vbox, 0);
 
 	hbox = gtk_hbox_new (0, 2);
@@ -744,7 +745,7 @@ key_dialog_show ()
 
 	vbox2 = gtk_vbox_new (0, 2);
 	gtk_box_pack_end (GTK_BOX (hbox), vbox2, 1, 1, 0);
-	wid = gtk_button_new_with_label (_("Add new"));
+	wid = gtk_button_new_with_label (_("Add New"));
 	gtk_box_pack_start (GTK_BOX (vbox2), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (key_dialog_add_new), list);
@@ -800,7 +801,7 @@ key_dialog_show ()
 	gtk_xtext_set_tint (GTK_XTEXT (wid), prefs.tint_red, prefs.tint_green, prefs.tint_blue);
 	gtk_xtext_set_background (GTK_XTEXT (wid),
 									  channelwin_pix,
-									  prefs.transparent, prefs.tint);
+									  prefs.transparent);
 	gtk_widget_set_usize (wid, 0, 75);
 	gtk_box_pack_start (GTK_BOX (hbox2), wid, 1, 1, 1);
 	gtk_xtext_set_font (GTK_XTEXT (wid), prefs.font_normal);
@@ -832,7 +833,7 @@ key_save_kbs (char *fn)
 	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
 	if (fd < 0)
 	{
-		gtkutil_simpledialog (_("Error opening keys config file\n"));
+		fe_message (_("Error opening keys config file\n"), FE_MSG_ERROR);
 		return;
 	}
 	write (fd, buf,
@@ -1005,7 +1006,7 @@ key_load_kbs (char *filename)
 				snprintf (ibuf, 1024,
 							 _("Unknown keyname %s in key bindings config file\nLoad aborted, please fix %s/keybindings.conf\n"),
 							 buf, get_xdir_utf8 ());
-				gtkutil_simpledialog (ibuf);
+				fe_message (ibuf, FE_MSG_ERROR);
 				free (ibuf);
 				return 2;
 			}
@@ -1043,7 +1044,7 @@ key_load_kbs (char *filename)
 				snprintf (ibuf, 1024,
 							 _("Unknown action %s in key bindings config file\nLoad aborted, Please fix %s/keybindings\n"),
 							 buf, get_xdir_utf8 ());
-				gtkutil_simpledialog (ibuf);
+				fe_message (ibuf, FE_MSG_ERROR);
 				free (ibuf);
 				return 3;
 			}
@@ -1064,7 +1065,7 @@ key_load_kbs (char *filename)
 				snprintf (ibuf, 1024,
 							 _("Expecting Data line (beginning Dx{:|!}) but got:\n%s\n\nLoad aborted, Please fix %s/keybindings\n"),
 							 buf, get_xdir_utf8 ());
-				gtkutil_simpledialog (ibuf);
+				fe_message (ibuf, FE_MSG_ERROR);
 				free (ibuf);
 				return 4;
 			}
@@ -1134,7 +1135,7 @@ key_load_kbs (char *filename)
 						_("Key bindings config file is corrupt, load aborted\n"
 								 "Please fix %s/keybindings.conf\n"),
 						 get_xdir_utf8 ());
-	gtkutil_simpledialog (ibuf);
+	fe_message (ibuf, FE_MSG_ERROR);
 	free (ibuf);
 	return 5;
 }
@@ -1393,6 +1394,19 @@ key_action_tab_clean(void)
 /* Used in the followig completers */
 #define COMP_BUF 2048
 
+/* For use in sorting the user list for completion */
+static int
+talked_recent_cmp (struct User *a, struct User *b)
+{
+	if (a->lasttalk < b->lasttalk)
+		return -1;
+
+	if (a->lasttalk > b->lasttalk)
+		return 1;
+
+	return 0;
+}
+
 static int
 key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 							struct session *sess)
@@ -1483,6 +1497,8 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 		{
 			gcomp = g_completion_new((GCompletionFunc)gcomp_nick_func);
 			tmp_list = userlist_double_list(sess); /* create a temp list so we can free the memory */
+			if (prefs.completion_sort == 1)	/* sort in last-talk order? */
+				tmp_list = g_list_sort (tmp_list, (void *)talked_recent_cmp);
 		}
 		else
 		{
@@ -1668,7 +1684,7 @@ static int
 key_action_move_tab_left (GtkWidget * wid, GdkEventKey * ent, char *d1,
 								  char *d2, struct session *sess)
 {
-	mg_move_tab (sess->res->tab, +1);
+	mg_move_tab (sess, +1);
 	return 2;						  /* don't allow default action */
 }
 
@@ -1676,7 +1692,7 @@ static int
 key_action_move_tab_right (GtkWidget * wid, GdkEventKey * ent, char *d1,
 									char *d2, struct session *sess)
 {
-	mg_move_tab (sess->res->tab, -1);
+	mg_move_tab (sess, -1);
 	return 2;						  /* -''- */
 }
 
@@ -1684,7 +1700,7 @@ static int
 key_action_move_tab_family_left (GtkWidget * wid, GdkEventKey * ent, char *d1,
 								  char *d2, struct session *sess)
 {
-	mg_move_tab_family (sess->res->tab, +1);
+	mg_move_tab_family (sess, +1);
 	return 2;						  /* don't allow default action */
 }
 
@@ -1692,7 +1708,7 @@ static int
 key_action_move_tab_family_right (GtkWidget * wid, GdkEventKey * ent, char *d1,
 									char *d2, struct session *sess)
 {
-	mg_move_tab_family (sess->res->tab, -1);
+	mg_move_tab_family (sess, -1);
 	return 2;						  /* -''- */
 }
 
