@@ -92,7 +92,7 @@ static PangoAttrList *plain_list = NULL;
 
 
 static PangoAttrList *
-mg_attr_list_create (GdkColor *col, int canbesmall)
+mg_attr_list_create (GdkColor *col, gboolean small)
 {
 	PangoAttribute *attr;
 	PangoAttrList *list;
@@ -107,7 +107,7 @@ mg_attr_list_create (GdkColor *col, int canbesmall)
 		pango_attr_list_insert (list, attr);
 	}
 
-	if (canbesmall && prefs.tab_small)
+	if (small)
 	{
 		attr = pango_attr_scale_new (PANGO_SCALE_SMALL);
 		attr->start_index = 0;
@@ -130,11 +130,11 @@ mg_create_tab_colors (void)
 		pango_attr_list_unref (away_list);
 	}
 
-	plain_list = mg_attr_list_create (NULL, 1);
-	newdata_list = mg_attr_list_create (&colors[COL_NEW_DATA], 1);
-	nickseen_list = mg_attr_list_create (&colors[COL_HILIGHT], 1);
-	newmsg_list = mg_attr_list_create (&colors[COL_NEW_MSG], 1);
-	away_list = mg_attr_list_create (&colors[COL_AWAY], 0);
+	plain_list = mg_attr_list_create (NULL, prefs.tab_small);
+	newdata_list = mg_attr_list_create (&colors[COL_NEW_DATA], prefs.tab_small);
+	nickseen_list = mg_attr_list_create (&colors[COL_HILIGHT], prefs.tab_small);
+	newmsg_list = mg_attr_list_create (&colors[COL_NEW_MSG], prefs.tab_small);
+	away_list = mg_attr_list_create (&colors[COL_AWAY], FALSE);
 }
 
 #ifdef WIN32
@@ -212,6 +212,7 @@ unflash_window (GtkWidget *win)
 void
 fe_set_tab_color (struct session *sess, int col, int flash)
 {
+	struct session *server_sess = sess->server->server_session;
 	if (sess->gui->is_tab && (col == 0 || sess != current_tab))
 	{
 		switch (col)
@@ -227,18 +228,45 @@ fe_set_tab_color (struct session *sess, int col, int flash)
 			sess->msg_said = FALSE;
 			sess->nick_said = FALSE;
 			chan_set_color (sess->res->tab, newdata_list);
+
+			if (chan_is_collapsed (sess->res->tab))
+			{
+				server_sess->new_data = TRUE;
+				server_sess->msg_said = FALSE;
+				server_sess->nick_said = FALSE;
+				chan_set_color (chan_get_parent (sess->res->tab), newdata_list);
+			}
+				
 			break;
 		case 2:	/* new message arrived in channel (light red) */
 			sess->new_data = FALSE;
 			sess->msg_said = TRUE;
 			sess->nick_said = FALSE;
 			chan_set_color (sess->res->tab, newmsg_list);
+			
+			if (chan_is_collapsed (sess->res->tab))
+			{
+				server_sess->new_data = FALSE;
+				server_sess->msg_said = TRUE;
+				server_sess->nick_said = FALSE;
+				chan_set_color (chan_get_parent (sess->res->tab), newmsg_list);
+			}
+			
 			break;
 		case 3:	/* your nick has been seen (blue) */
 			sess->new_data = FALSE;
 			sess->msg_said = FALSE;
 			sess->nick_said = TRUE;
 			chan_set_color (sess->res->tab, nickseen_list);
+
+			if (chan_is_collapsed (sess->res->tab))
+			{
+				server_sess->new_data = FALSE;
+				server_sess->msg_said = FALSE;
+				server_sess->nick_said = TRUE;
+				chan_set_color (chan_get_parent (sess->res->tab), nickseen_list);
+			}
+				
 			break;
 		}
 	}
@@ -253,7 +281,7 @@ static void
 mg_set_myself_away (session_gui *gui, gboolean away)
 {
 	gtk_label_set_attributes (GTK_LABEL (GTK_BIN (gui->nick_label)->child),
-									  away ? away_list : plain_list);
+									  away ? away_list : NULL);
 }
 
 /* change the little icon to the left of your nickname */
@@ -798,6 +826,7 @@ mg_populate (session *sess)
 	/* menu items */
 	GTK_CHECK_MENU_ITEM (gui->menu_item[MENU_ID_AWAY])->active = sess->server->is_away;
 	gtk_widget_set_sensitive (gui->menu_item[MENU_ID_AWAY], sess->server->connected);
+	gtk_widget_set_sensitive (gui->menu_item[MENU_ID_JOIN], sess->server->end_of_motd);
 	gtk_widget_set_sensitive (gui->menu_item[MENU_ID_DISCONNECT],
 									  sess->server->connected || sess->server->recondelay_tag);
 
@@ -1144,26 +1173,24 @@ mg_link_gentab (chan *ch, GtkWidget *box)
 }
 
 static void
-mg_detach_tab_cb (GtkWidget *item, session *sess)
+mg_detach_tab_cb (GtkWidget *item, chan *ch)
 {
-	chan *ch = chanview_get_focused (mg_gui->chanview);
-
 	if (chan_get_tag (ch) == TAG_IRC)	/* IRC tab */
 	{
-		mg_link_irctab (sess, 1);
+		/* userdata is session * */
+		mg_link_irctab (chan_get_userdata (ch), 1);
 		return;
 	}
 
-	mg_link_gentab (ch, (GtkWidget *) sess);	/* non-IRC tab */
+	/* userdata is GtkWidget * */
+	mg_link_gentab (ch, chan_get_userdata (ch));	/* non-IRC tab */
 }
 
 static void
-mg_destroy_tab_cb (GtkWidget *item, session *sess)
+mg_destroy_tab_cb (GtkWidget *item, chan *ch)
 {
-	chan *ch = chanview_get_focused (mg_gui->chanview);
-
 	/* treat it just like the X button press */
-	mg_xbutton_cb (mg_gui->chanview, ch, chan_get_tag (ch), sess);
+	mg_xbutton_cb (mg_gui->chanview, ch, chan_get_tag (ch), chan_get_userdata (ch));
 }
 
 static void
@@ -1310,10 +1337,10 @@ mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventBut
 	gtk_widget_show (item);
 
 	mg_create_icon_item (_("_Close Tab"), GTK_STOCK_CLOSE, menu,
-								mg_destroy_tab_cb, sess);
+								mg_destroy_tab_cb, ch);
 	/*if (tag == TAG_IRC)*/
 		mg_create_icon_item (_("_Detach Tab"), GTK_STOCK_REDO, menu,
-									mg_detach_tab_cb, sess);
+									mg_detach_tab_cb, ch);
 
 	if (sess && tabmenu_list)
 		menu_create (menu, tabmenu_list, sess->channel, FALSE);
@@ -2163,7 +2190,7 @@ mg_create_meters (session_gui *gui, GtkWidget *parent_box)
 	GtkWidget *infbox, *wid, *box;
 
 	gui->meter_box = infbox = box = gtk_vbox_new (0, 1);
-	gtk_box_pack_end (GTK_BOX (parent_box), box, 0, 0, 0);
+	gtk_box_pack_start (GTK_BOX (parent_box), box, 0, 0, 0);
 
 	if ((prefs.lagometer & 2) || (prefs.throttlemeter & 2))
 	{
@@ -3058,6 +3085,8 @@ mg_set_title (GtkWidget *vbox, char *title) /* for non-irc tab/window only */
 void
 fe_server_callback (server *serv)
 {
+	joind (1, serv);
+
 	if (serv->gui->chanlist_window)
 		mg_close_gen (NULL, serv->gui->chanlist_window);
 
