@@ -1,11 +1,15 @@
-extern GdkPixbuf *pix_about;
-extern GdkPixbuf *pix_xchat;
 extern GdkPixbuf *pix_book;
+extern GdkPixbuf *pix_hop;
 extern GdkPixbuf *pix_purple;
 extern GdkPixbuf *pix_red;
 extern GdkPixbuf *pix_op;
-extern GdkPixbuf *pix_hop;
 extern GdkPixbuf *pix_voice;
+extern GdkPixbuf *pix_xchat;
+
+extern GdkPixbuf *pix_channel;
+extern GdkPixbuf *pix_dialog;
+extern GdkPixbuf *pix_server;
+extern GdkPixbuf *pix_util;
 
 extern GdkPixmap *pixmap_load_from_file (char *file);
 extern void pixmaps_init (void);
