@@ -10,11 +10,7 @@
 #include <gtk/gtkversion.h>
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtkcellrenderertext.h>
-#if GTK_CHECK_VERSION(2,4,0)
 #include <gtk/gtkcomboboxentry.h>
-#else
-#include <gtk/gtkcombo.h>
-#endif
 #include <gtk/gtkentry.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkhbbox.h>
@@ -40,6 +36,7 @@
 #include "../common/xchatc.h"
 #include "../common/servlist.h"
 #include "../common/cfgfiles.h"
+#include "../common/fe.h"
 
 #include "fe-gtk.h"
 #include "gtkutil.h"
@@ -62,6 +59,7 @@ static GtkWidget *entry_greal;
 /* edit area */
 static GtkWidget *edit_win;
 static GtkWidget *edit_entry_nick;
+static GtkWidget *edit_entry_nick2;
 static GtkWidget *edit_entry_user;
 static GtkWidget *edit_entry_real;
 static GtkWidget *edit_entry_join;
@@ -69,6 +67,7 @@ static GtkWidget *edit_entry_pass;
 static GtkWidget *edit_entry_cmd;
 static GtkWidget *edit_entry_nickserv;
 static GtkWidget *edit_label_nick;
+static GtkWidget *edit_label_nick2;
 static GtkWidget *edit_label_real;
 static GtkWidget *edit_label_user;
 static GtkWidget *edit_tree;
@@ -343,9 +342,52 @@ servlist_move_network (ircnet *net, int delta)
 	}
 }
 
+#if 0
 static gboolean
-servlist_net_keypress_cb (GtkWidget *wid, GdkEventKey *evt, gpointer userdata)
+servlist_auto_find (GtkWidget *wid, GdkEventKey *evt, gpointer tree)
 {
+	GtkTreeModel *model = gtk_tree_view_get_model (tree);
+	GtkTreeIter iter;
+	unsigned char c;
+	unsigned char *net_name;
+
+	if (evt->keyval > 0x7a || evt->keyval < 0x41)
+		return FALSE;
+
+	c = toupper (evt->keyval);
+
+	/* scroll to a network that starts with the letter pressed */
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		do
+		{
+			gtk_tree_model_get (model, &iter, 0, &net_name, -1);
+			if (net_name)
+			{
+				if (toupper (net_name[0]) == c)
+				{
+					servlist_select_and_show (tree, &iter, GTK_LIST_STORE (model));
+					g_free (net_name);
+					return TRUE;
+				}
+				g_free (net_name);
+			}
+		}
+		while (gtk_tree_model_iter_next (model, &iter));
+	}
+
+	return FALSE;
+}
+#endif
+
+static gboolean
+servlist_net_keypress_cb (GtkWidget *wid, GdkEventKey *evt, gpointer tree)
+{
+#if 0		/* GTK's auto-find does this better */
+	if (servlist_auto_find (wid, evt, tree))
+		return TRUE;
+#endif
+
 	if (!selected_net)
 		return FALSE;
 
@@ -440,6 +482,7 @@ servlist_edit_close_cb (GtkWidget *button, gpointer userdata)
 	ircnet *net = selected_net;
 
 	servlist_update_from_entry (&net->nick, edit_entry_nick);
+	servlist_update_from_entry (&net->nick2, edit_entry_nick2);
 	servlist_update_from_entry (&net->user, edit_entry_user);
 	servlist_update_from_entry (&net->real, edit_entry_real);
 
@@ -561,7 +604,7 @@ servlist_find_selected_net (GtkTreeSelection *sel)
 	if (gtk_tree_selection_get_selected (sel, &model, &iter))
 	{
 		gtk_tree_model_get (model, &iter, 0, &netname, -1);
-		net = servlist_net_find (netname, &pos);
+		net = servlist_net_find (netname, &pos, strcmp);
 		g_free (netname);
 		if (net)
 			prefs.slist_select = pos;
@@ -607,25 +650,6 @@ servlist_savegui (void)
 	return 0;
 }
 
-static void
-servlist_connectnew_cb (GtkWidget *button, gpointer userdata)
-{
-	if (!selected_net)
-		return;
-
-	if (servlist_savegui () != 0)
-	{
-		gtkutil_simpledialog (_("User name and Real name cannot be left blank."));
-		return;
-	}
-
-	/* give it a NULL sess and it'll open a new tab for us */
-	servlist_connect (NULL, selected_net);
-
-	gtk_widget_destroy (serverlist_win);
-	serverlist_win = NULL;
-}
-
 static void
 servlist_connect_cb (GtkWidget *button, gpointer userdata)
 {
@@ -634,14 +658,16 @@ servlist_connect_cb (GtkWidget *button, gpointer userdata)
 
 	if (servlist_savegui () != 0)
 	{
-		gtkutil_simpledialog (_("User name and Real name cannot be left blank."));
+		fe_message (_("User name and Real name cannot be left blank."), FE_MSG_ERROR);
 		return;
 	}
 
 	if (!is_session (servlist_sess))
 		servlist_sess = NULL;
+	else if (servlist_sess->server->connected)
+		servlist_sess = NULL;
 
-	servlist_connect (servlist_sess, selected_net);
+	servlist_connect (servlist_sess, selected_net, TRUE);
 
 	gtk_widget_destroy (serverlist_win);
 	serverlist_win = NULL;
@@ -660,7 +686,7 @@ servlist_celledit_cb (GtkCellRendererText *cell, gchar *arg1, gchar *arg2,
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_model_get (model, &iter, 0, &netname, -1);
 
-	net = servlist_net_find (netname, NULL);
+	net = servlist_net_find (netname, NULL, strcmp);
 	g_free (netname);
 	if (net)
 	{
@@ -711,6 +737,9 @@ servlist_check_cb (GtkWidget *but, gpointer num_p)
 			gtk_widget_hide (edit_label_nick);
 			gtk_widget_hide (edit_entry_nick);
 
+			gtk_widget_hide (edit_label_nick2);
+			gtk_widget_hide (edit_entry_nick2);
+
 			gtk_widget_hide (edit_label_user);
 			gtk_widget_hide (edit_entry_user);
 
@@ -721,6 +750,9 @@ servlist_check_cb (GtkWidget *but, gpointer num_p)
 			gtk_widget_show (edit_label_nick);
 			gtk_widget_show (edit_entry_nick);
 
+			gtk_widget_show (edit_label_nick2);
+			gtk_widget_show (edit_entry_nick2);
+
 			gtk_widget_show (edit_label_user);
 			gtk_widget_show (edit_entry_user);
 
@@ -857,12 +889,7 @@ servlist_create_charsetcombo (void)
 {
 	GtkWidget *cb;
 	int i;
-#if GTK_CHECK_VERSION(2,4,0)
-#else
-	static GList *cbitems = NULL;
-#endif
 
-#if GTK_CHECK_VERSION(2,4,0)
 	cb = gtk_combo_box_entry_new_text ();
 	gtk_combo_box_append_text (GTK_COMBO_BOX (cb), "System default");
 	i = 0;
@@ -873,24 +900,6 @@ servlist_create_charsetcombo (void)
 	}
 	g_signal_connect (G_OBJECT (GTK_BIN (cb)->child), "changed",
 							G_CALLBACK (servlist_combo_cb), NULL);
-#else
-	/* === OLD GTK 2.0/2.2 code === */
-	if (cbitems == NULL)
-	{
-		cbitems = g_list_append (cbitems, "System default");
-		i = 0;
-		while (pages[i])
-		{
-			cbitems = g_list_append (cbitems, (char *)pages[i]);
-			i++;
-		}
-	}
-
-	cb = gtk_combo_new ();
-	gtk_combo_set_popdown_strings (GTK_COMBO (cb), cbitems);
-	g_signal_connect (G_OBJECT (GTK_COMBO (cb)->entry), "changed",
-							G_CALLBACK (servlist_combo_cb), NULL);
-#endif
 
 	return cb;
 }
@@ -904,42 +913,6 @@ no_servlist (GtkWidget * igad, gpointer serv)
 		prefs.slist_skip = FALSE;
 }
 
-static gboolean
-servlist_key_cb (GtkWidget *wid, GdkEventKey *evt, gpointer tree)
-{
-	GtkTreeModel *model = gtk_tree_view_get_model (tree);
-	GtkTreeIter iter;
-	unsigned char c;
-	unsigned char *net_name;
-
-	if (evt->keyval > 0x7a || evt->keyval < 0x41)
-		return FALSE;
-
-	c = toupper (evt->keyval);
-
-	/* scroll to a network that starts with the letter pressed */
-	if (gtk_tree_model_get_iter_first (model, &iter))
-	{
-		do
-		{
-			gtk_tree_model_get (model, &iter, 0, &net_name, -1);
-			if (net_name)
-			{
-				if (toupper (net_name[0]) == c)
-				{
-					servlist_select_and_show (tree, &iter, GTK_LIST_STORE (model));
-					g_free (net_name);
-					return TRUE;
-				}
-				g_free (net_name);
-			}
-		}
-		while (gtk_tree_model_iter_next (model, &iter));
-	}
-
-	return FALSE;
-}
-
 static GtkWidget *
 bold_label (char *text)
 {
@@ -985,7 +958,7 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 
 	editwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width (GTK_CONTAINER (editwindow), 4);
-	snprintf (buf, sizeof (buf), _("X-Chat: Edit %s"), net->name);
+	snprintf (buf, sizeof (buf), _("XChat: Edit %s"), net->name);
 	gtk_window_set_title (GTK_WINDOW (editwindow), buf);
 	gtk_window_set_default_size (GTK_WINDOW (editwindow), 354, 0);
 	gtk_window_set_position (GTK_WINDOW (editwindow), GTK_WIN_POS_MOUSE);
@@ -1030,72 +1003,73 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 		servlist_create_entry (table3, _("_Nick name:"), 5, net->nick,
 									  &edit_label_nick, 0);
 
+	edit_entry_nick2 =
+		servlist_create_entry (table3, _("Second choice:"), 6, net->nick2,
+									  &edit_label_nick2, 0);
+
 	edit_entry_user =
-		servlist_create_entry (table3, _("_User name:"), 6, net->user,
+		servlist_create_entry (table3, _("_User name:"), 7, net->user,
 									  &edit_label_user, 0);
 
 	edit_entry_real =
-		servlist_create_entry (table3, _("Real na_me:"), 7, net->real,
+		servlist_create_entry (table3, _("Rea_l name:"), 8, net->real,
 									  &edit_label_real, 0);
 
 	label21 = bold_label (_("Connecting"));
-	gtk_table_attach (GTK_TABLE (table3), label21, 0, 3, 8, 9,
+	gtk_table_attach (GTK_TABLE (table3), label21, 0, 3, 9, 10,
 							(GtkAttachOptions) (GTK_FILL),
 							(GtkAttachOptions) (0), 0, 3);
 
 	servlist_create_check (3, net->flags & FLAG_AUTO_CONNECT, table3,
-								  9, 1, _("Auto connect to this network at startup"));
+								  11, 1, _("Auto connect to this network at startup"));
 	servlist_create_check (4, net->flags & FLAG_USE_PROXY, table3,
-								  10, 1, _("Use a proxy server"));
+								  12, 1, _("Use a proxy server"));
 	check = servlist_create_check (2, net->flags & FLAG_USE_SSL, table3,
-								  11, 1, _("Use SSL for all the servers on this network"));
+								  13, 1, _("Use SSL for all the servers on this network"));
 #ifndef USE_OPENSSL
 	gtk_widget_set_sensitive (check, FALSE);
 #endif
 	check = servlist_create_check (5, net->flags & FLAG_ALLOW_INVALID, table3,
-								  12, 1, _("Accept invalid SSL certificate"));
+								  14, 1, _("Accept invalid SSL certificate"));
 #ifndef USE_OPENSSL
 	gtk_widget_set_sensitive (check, FALSE);
 #endif
 
 	edit_entry_join =
-		servlist_create_entry (table3, _("C_hannels to join:"), 13,
+		servlist_create_entry (table3, _("C_hannels to join:"), 15,
 									  net->autojoin, 0,
 				  _("Channels to join, separated by commas, but not spaces!"));
 
 	edit_entry_cmd =
-		servlist_create_entry (table3, _("Connect command:"), 14,
+		servlist_create_entry (table3, _("Connect command:"), 16,
 									  net->command, 0,
 					_("Extra command to execute after connecting. If you need more than one, set this to LOAD -e <filename>, where <filename> is a text-file full of commands to execute."));
 
 	edit_entry_nickserv =
-		servlist_create_entry (table3, _("Nickserv password:"), 15,
-									  net->nickserv, 0, 0);
+		servlist_create_entry (table3, _("Nickserv password:"), 17,
+									  net->nickserv, 0,
+					_("If your nickname requires a password, enter it here. Not all IRC networks support this."));
 	gtk_entry_set_visibility (GTK_ENTRY (edit_entry_nickserv), FALSE);
 
 	edit_entry_pass =
-		servlist_create_entry (table3, _("Server password:"), 16,
+		servlist_create_entry (table3, _("Server password:"), 18,
 									  net->pass, 0,
 					_("Password for the server, if in doubt, leave blank."));
 	gtk_entry_set_visibility (GTK_ENTRY (edit_entry_pass), FALSE);
 
 	label34 = gtk_label_new (_("Character set:"));
 	gtk_widget_show (label34);
-	gtk_table_attach (GTK_TABLE (table3), label34, 1, 2, 17, 18,
+	gtk_table_attach (GTK_TABLE (table3), label34, 1, 2, 19, 20,
 							(GtkAttachOptions) (GTK_FILL),
 							(GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label34), 0, 0.5);
 
 	comboboxentry_charset = servlist_create_charsetcombo ();
 	ignore_changed = TRUE;
-#if GTK_CHECK_VERSION(2,4,0)
 	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (comboboxentry_charset)->child), net->encoding ? net->encoding : "System default");
-#else
-	gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (comboboxentry_charset)->entry), net->encoding ? net->encoding : "System default");
-#endif
 	ignore_changed = FALSE;
 	gtk_widget_show (comboboxentry_charset);
-	gtk_table_attach (GTK_TABLE (table3), comboboxentry_charset, 2, 3, 17, 18,
+	gtk_table_attach (GTK_TABLE (table3), comboboxentry_charset, 2, 3, 19, 20,
 							(GtkAttachOptions) (GTK_FILL),
 							(GtkAttachOptions) (GTK_FILL), 0, 0);
 
@@ -1134,6 +1108,7 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 								NULL);
 
 	vbuttonbox1 = gtk_vbutton_box_new ();
+	gtk_box_set_spacing (GTK_BOX (vbuttonbox1), 3);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (vbuttonbox1), GTK_BUTTONBOX_START);
 	gtk_widget_show (vbuttonbox1);
 	gtk_box_pack_start (GTK_BOX (hbox1), vbuttonbox1, FALSE, FALSE, 3);
@@ -1181,6 +1156,9 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 		gtk_widget_hide (edit_label_nick);
 		gtk_widget_hide (edit_entry_nick);
 
+		gtk_widget_hide (edit_label_nick2);
+		gtk_widget_hide (edit_entry_nick2);
+
 		gtk_widget_hide (edit_label_user);
 		gtk_widget_hide (edit_entry_user);
 
@@ -1225,7 +1203,6 @@ servlist_open_networks (void)
 	GtkWidget *hseparator1;
 	GtkWidget *hbuttonbox1;
 	GtkWidget *button_connect;
-	GtkWidget *button_connectnew;
 	GtkWidget *button_close;
 	GtkTreeModel *model;
 	GtkListStore *store;
@@ -1233,11 +1210,11 @@ servlist_open_networks (void)
 
 	servlist = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width (GTK_CONTAINER (servlist), 4);
-	gtk_window_set_title (GTK_WINDOW (servlist), _("X-Chat: Server List"));
+	gtk_window_set_title (GTK_WINDOW (servlist), _("XChat: Server List"));
 #ifdef WIN32
-	gtk_window_set_default_size (GTK_WINDOW (servlist), 374, 426);
+	gtk_window_set_default_size (GTK_WINDOW (servlist), 324, 426);
 #else
-	gtk_window_set_default_size (GTK_WINDOW (servlist), 414, 478);
+	gtk_window_set_default_size (GTK_WINDOW (servlist), 364, 478);
 #endif
 	gtk_window_set_position (GTK_WINDOW (servlist), GTK_WIN_POS_MOUSE);
 	gtk_window_set_role (GTK_WINDOW (servlist), "servlist");
@@ -1385,6 +1362,7 @@ servlist_open_networks (void)
 							(GtkAttachOptions) (0), 0, 0);
 
 	vbuttonbox2 = gtk_vbutton_box_new ();
+	gtk_box_set_spacing (GTK_BOX (vbuttonbox2), 3);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (vbuttonbox2), GTK_BUTTONBOX_START);
 	gtk_widget_show (vbuttonbox2);
 	gtk_table_attach (GTK_TABLE (table4), vbuttonbox2, 1, 2, 0, 1,
@@ -1435,23 +1413,10 @@ servlist_open_networks (void)
 	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button_close);
 	GTK_WIDGET_SET_FLAGS (button_close, GTK_CAN_DEFAULT);
 
-	button_connect = gtk_button_new_with_mnemonic (_("C_onnect"));
-	gtk_widget_show (button_connect);
-	g_signal_connect (G_OBJECT (button_connect), "clicked",
-							G_CALLBACK (servlist_connect_cb), 0);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button_connect);
+	button_connect = gtkutil_button (hbuttonbox1, GTK_STOCK_CONNECT, NULL,
+												servlist_connect_cb, NULL, _("C_onnect"));
 	GTK_WIDGET_SET_FLAGS (button_connect, GTK_CAN_DEFAULT);
 
-	if (servlist_sess)
-	{
-		button_connectnew = gtk_button_new_with_mnemonic (_("Connect in new tab"));
-		gtk_widget_show (button_connectnew);
-		g_signal_connect (G_OBJECT (button_connectnew), "clicked",
-								G_CALLBACK (servlist_connectnew_cb), 0);
-		gtk_container_add (GTK_CONTAINER (hbuttonbox1), button_connectnew);
-		GTK_WIDGET_SET_FLAGS (button_connectnew, GTK_CAN_DEFAULT);
-	}
-
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label3), entry1);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label6), entry4);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label7), entry5);
@@ -1482,9 +1447,7 @@ fe_serverlist_open (session *sess)
 	g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (networks_tree))),
 							"changed", G_CALLBACK (servlist_network_row_cb), NULL);
 	g_signal_connect (G_OBJECT (networks_tree), "key_press_event",
-							G_CALLBACK (servlist_key_cb), networks_tree);
-	g_signal_connect (G_OBJECT (networks_tree), "key_press_event",
-							G_CALLBACK (servlist_net_keypress_cb), 0);
+							G_CALLBACK (servlist_net_keypress_cb), networks_tree);
 
 	gtk_widget_show (serverlist_win);
 }
